#[macro_use] extern crate log;
extern crate env_logger;
extern crate portaudio;
extern crate soundchip;

use std::error::Error;
use std::sync::mpsc;
use portaudio as pa;

// TODO: replace with config struct, loaded via Serde
const CHANNELS: u32 = 2;
const SAMPLE_RATE: f64 = 44_100.0; // 44.1kHz
const FRAMES_PER_BUFFER: u32 = 64;

fn run() -> Result<(), Box<Error>> {
    env_logger::init().unwrap();

    info!("Portaudio launching!");
    let pa = pa::PortAudio::new()?;
    info!("PortAudio:");
    info!("version: {}, {}", pa.version(), pa.version_text().unwrap());

    let (ctx, crx) = mpsc::channel();
    let mut chip = soundchip::Chip::new(crx);

    let pa_settings = pa.default_output_stream_settings(CHANNELS as i32, SAMPLE_RATE, FRAMES_PER_BUFFER)?;
    let pa_callback = move |pa::OutputStreamCallbackArgs {mut buffer, frames, ..}| {
        chip.render_frames(&mut buffer, frames, SAMPLE_RATE, CHANNELS);
        pa::Continue
    };

    let mut pa_stream = pa.open_non_blocking_stream(pa_settings, pa_callback)?;
    pa_stream.start()?;

    loop {
        pa.sleep(1_000);
    }

    Ok(())
}

fn main() {
    if let Err(err) = run() {
        error!("{:?}", err);
    }
}
