use ::waveform::Waveform;
use ::audio::{Frame, Renderable, from_bitamp, from_bitpan};

#[derive(Default)]
pub struct Channel<W> where W: Waveform {
    waveform: W,
    phase: f64,

    pub frequency: f64,

    pan: f32,
    amp: f32,
}

fn next_sample_phase(phase: f64, frequency: f64, sample_rate: f64) -> f64 {
    phase + (frequency / sample_rate)
}

impl<W> Channel<W> where W: Waveform + Default {
    pub fn new(wavf: W, f0: f64, a0: f32, p0: f32) -> Channel<W> {
        Channel {
            waveform: wavf,
            phase: 0.,
            frequency: f0,
            pan: p0,
            amp: a0,
        }
    }

    pub fn set_pan(&mut self, angle: i8) {
        self.pan = from_bitpan(angle);
    }

    pub fn set_amp(&mut self, amp: u8) {
        self.amp = from_bitamp(amp);
    }

    pub fn note_on(&mut self, note: u8) {}
    pub fn note_off(&mut self, note: u8) {}
}

impl<W> Renderable for Channel<W> where W: Waveform {
    fn render_sample(&mut self, sample_rate: f64) -> Frame {
        let x = Frame::mono_to_stereo(self.waveform.x(self.phase), self.pan);

        self.phase = next_sample_phase(self.phase, self.frequency, sample_rate);

        x * self.amp
    }
}
