mod channels;
mod waveform;
mod audio;

use audio::{Frame, Renderable};
use channels::Channel;
use waveform::{Waveform, Saw, Square};
use std::sync::mpsc::Receiver;

pub struct Chip {
    pub vol: f32,
    rx: Receiver<Message>,

    renderables: Vec<Box<Renderable>>,
}

pub enum Message {
    NoteOn(usize, u8),
    NoteOff(usize, u8),
}


impl Chip {
    pub fn new(mpi_rx: Receiver<Message>) -> Chip {
        Chip {
            vol: 0.25,
            rx: mpi_rx,
            renderables: Vec::with_capacity(8),
        }
    }

    fn process_messages(&mut self) {
        match self.rx.try_recv() {
            Ok(m) => match m {
                Message::NoteOn(channel, note) => {
                }
                Message::NoteOff(channel, note) => {
                }
            },
            Err(_) => {} // nop if there's nothing pending in the channel
        }
    }

    pub fn render_sample(&mut self, sample_rate: f64) -> Frame {
        self.renderables.iter_mut()
            .map(|m| m.render_sample(sample_rate))
            .sum()
    }

    pub fn render_frames(&mut self, buffer: &mut [f32], n_frames: usize,
                        sample_rate: f64, _channels: u32) {
        let mut idx = 0;
        for _ in 0..n_frames {
            self.process_messages();

            let sample = self.render_sample(sample_rate);
            buffer[idx] = sample.l;
            buffer[idx+1] = sample.r;
            idx += 2;
        }
    }
}
